Class: Kramdown::Converter::Rfc2629

Inherits:
Base
  • Object
show all
Includes:
Utils::Html
Defined in:
lib/kramdown-rfc2629.rb

Overview

Converts a Kramdown::Document to HTML.

Constant Summary collapse

KRAMDOWN_PERSISTENT =

:stopdoc:

KRAMDOWN_PERSISTENT_VERBOSE =
/v/ === KRAMDOWN_PERSISTENT
INDENTATION =

Defines the amount of indentation used when nesting XML tags.

2
SVG_COLORS =
Hash.new {|h, k| k}
B_W_THRESHOLD =

a little brighter than 1/2 0xFF -> white

hex_to_lin("a4")
SVG_NAMESPACES =
{"xmlns"=>"http://www.w3.org/2000/svg",
"xlink"=>"http://www.w3.org/1999/xlink"}
DEFAULT_AASVG =
"aasvg --spaces=1"
ARTWORK_TYPES =
%w(ascii-art binary-art call-flow hex-dump svg)
STYLES =
{ul: 'symbols', ol: 'numbers', dl: 'hanging'}
HTML_TAGS_WITH_BODY =
['div', 'script']
ALIGNMENTS =
{ default: :left, left: :left, right: :right, center: :center}
COLS_ALIGN =
{ "l" => :left, "c" => :center, "r" => :right}
REFCACHEDIR =
ENV["KRAMDOWN_REFCACHEDIR"] || ".refcache"
KRAMDOWN_OFFLINE =

warn “*** REFCACHEDIR #REFCACHEDIR

KRAMDOWN_REFCACHE_REFETCH =
KRAMDOWN_REFCACHE_QUIET =
KRAMDOWN_REFCACHETTL =
(e = ENV["KRAMDOWN_REFCACHETTL"]) ? e.to_i : 3600
KRAMDOWN_REFCACHETTL_RFC =
(e = ENV["KRAMDOWN_REFCACHETTL_RFC"]) ? e.to_i : 86400*7
KRAMDOWN_REFCACHETTL_DOI_IANA =
(e = ENV["KRAMDOWN_REFCACHETTL_DOI_IANA"]) ? e.to_i : 86400
KRAMDOWN_REFCACHETTL_DOI =
(e = ENV["KRAMDOWN_REFCACHETTL_DOI"]) ? e.to_i : KRAMDOWN_REFCACHETTL_DOI_IANA
KRAMDOWN_REFCACHETTL_IANA =
(e = ENV["KRAMDOWN_REFCACHETTL_IANA"]) ? e.to_i : KRAMDOWN_REFCACHETTL_DOI_IANA
XML_RESOURCE_ORG_MAP =
subdirectory name, cache ttl in seconds, does it provide for ?anchor=
{
  "RFC" => ["bibxml", KRAMDOWN_REFCACHETTL_RFC, false,
            ->(fn, n){ [name = "reference.RFC.#{"%04d" % n.to_i}.xml",
                        "https://bib.ietf.org/public/rfc/bibxml/#{name}"] }
# was                         "https://www.rfc-editor.org/refs/bibxml/#{name}"] }
           ],
  "I-D" => ["bibxml3", false, false,
            ->(fn, n){ [fn,
                        "https://datatracker.ietf.org/doc/bibxml3/draft-#{n.sub(/\Adraft-/, '')}.xml"] }
           ],
  "BCP" => ["bibxml-rfcsubseries", KRAMDOWN_REFCACHETTL_RFC, false,
            ->(fn, n){ Rfc2629::bcp_std_ref("BCP", n) }
           ],
  "STD" => ["bibxml-rfcsubseries", KRAMDOWN_REFCACHETTL_RFC, false,
            ->(fn, n){ Rfc2629::bcp_std_ref("STD", n) }
           ],
  "W3C" => "bibxml4",
  "3GPP" => "bibxml5",
  "SDO-3GPP" => "bibxml5",
  "ANSI" => "bibxml2",
  "CCITT" => "bibxml2",
  "FIPS" => "bibxml2",
  # "IANA" => "bibxml2",   overtaken by bibxml8
  "IEEE" => "bibxml6",    # copied over to bibxml6 2019-02-27
  "ISO" => "bibxml2",
  "ITU" => "bibxml2",
  "NIST" => "bibxml2",
  "OASIS" => "bibxml2",
  "PKCS" => "bibxml2",
  "DOI" => ["bibxml7", KRAMDOWN_REFCACHETTL_DOI, true,
            ->(fn, n){ ["computed-#{fn}", [:DOI, n] ] }, true # always_altproc
           ], # emulate old 24 h cache
  "IANA" => ["bibxml8", KRAMDOWN_REFCACHETTL_IANA, true], # ditto
}
XML_RESOURCE_ORG_HOST =

XML_RESOURCE_ORG_HOST = ENV || “xml.resource.org” XML_RESOURCE_ORG_HOST = ENV || “xml2rfc.tools.ietf.org”

ENV["XML_RESOURCE_ORG_HOST"] || "bib.ietf.org"
XML_RESOURCE_ORG_PREFIX =
ENV["XML_RESOURCE_ORG_PREFIX"] ||
"https://#{XML_RESOURCE_ORG_HOST}/public/rfc"
KRAMDOWN_USE_TOOLS_SERVER =
KRAMDOWN_NO_TARGETS =
KRAMDOWN_KEEP_TARGETS =
EMPH =
{ em: "emph", strong: "strong"}
TYPOGRAPHIC_SYMS =
{
  :mdash => [::Kramdown::Utils::Entities.entity('mdash')],
  :ndash => [::Kramdown::Utils::Entities.entity('ndash')],
  :hellip => [::Kramdown::Utils::Entities.entity('hellip')],
  :laquo_space => [::Kramdown::Utils::Entities.entity('laquo'), ::Kramdown::Utils::Entities.entity('nbsp')],
  :raquo_space => [::Kramdown::Utils::Entities.entity('nbsp'), ::Kramdown::Utils::Entities.entity('raquo')],
  :laquo => [::Kramdown::Utils::Entities.entity('laquo')],
  :raquo => [::Kramdown::Utils::Entities.entity('raquo')]
}
MATH_LATEX_FILENAME =
File.expand_path '../../data/math.json', __FILE__
MATH_LATEX =
JSON.parse(File.read(MATH_LATEX_FILENAME, encoding: Encoding::UTF_8))
MATH_REPLACEMENTS =
MATH_COMBININGMARKS =
ITEM_RE =
'\s*(?:"([^"]*)"|([^,]*?))\s*'
IREF_RE =
%r{\A(!\s*)?#{ITEM_RE}(?:,#{ITEM_RE})?\z}

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(*doc) ⇒ Rfc2629

Initialize the XML converter with the given Kramdown document doc.



385
386
387
388
389
390
391
392
# File 'lib/kramdown-rfc2629.rb', line 385

def initialize(*doc)
  super
  @sec_level = 1
  @location_delta = 100000 # until reset
  @location_correction = 0 # pre-scanning corrections
  @in_dt = 0
  @footnote_names_in_use = {}
end

Class Method Details

.bcp_std_ref(t, n) ⇒ Object



1321
1322
1323
1324
1325
# File 'lib/kramdown-rfc2629.rb', line 1321

def self.bcp_std_ref(t, n)
  warn "*** #{t} anchors not supported in v2 format" unless $options.v3
  [name = "reference.#{t}.#{"%04d" % n.to_i}.xml",
   "#{XML_RESOURCE_ORG_PREFIX}/bibxml-rfcsubseries/#{name}"] # FOR NOW
end

.hex_to_lin(h) ⇒ Object



488
489
490
# File 'lib/kramdown-rfc2629.rb', line 488

def self.hex_to_lin(h)
  h.to_i(16)**2.22        # approximating sRGB gamma
end

.process_markdown(v) ⇒ Object



455
456
457
# File 'lib/kramdown-rfc2629.rb', line 455

def self.process_markdown(v)
  process_markdown1(v)[3..-6] # skip <t>...</t>\n
end

.process_markdown1(v) ⇒ Object

Uuh. Heavy coupling.



449
450
451
452
453
# File 'lib/kramdown-rfc2629.rb', line 449

def self.process_markdown1(v)             # Uuh.  Heavy coupling.
  doc = ::Kramdown::Document.new(v, $global_markdown_options)
  $stderr.puts doc.warnings.to_yaml unless doc.warnings.empty?
  doc.to_rfc2629
end

.process_markdown_to_rexml(v) ⇒ Object



459
460
461
462
# File 'lib/kramdown-rfc2629.rb', line 459

def self.process_markdown_to_rexml(v)
  s = process_markdown1(v)
  REXML::Document.new(s)
end

Instance Method Details

#capture_croak(t, err) ⇒ Object



567
568
569
570
571
572
573
# File 'lib/kramdown-rfc2629.rb', line 567

def capture_croak(t, err)
  if err != ''
    err.lines do |l|
      warn "*** [#{t}:] #{l.chomp}"
    end
  end
end

#clean_pcdata(parts) ⇒ Object

hack, will become unnecessary with XML2RFCv3



895
896
897
898
899
900
901
902
903
904
905
# File 'lib/kramdown-rfc2629.rb', line 895

def clean_pcdata(parts)    # hack, will become unnecessary with XML2RFCv3
  clean = ''
  irefs = ''
  # warn "clean_parts: #{parts.inspect}"
  parts.each do |p|
    md = p.match(%r{([^<]*)(.*)})
    clean << md[1]
    irefs << md[2]        # breaks for spanx... don't emphasize in headings!
  end
  [clean, irefs]
end

#clean_pcdatav3(parts) ⇒ Object

hack, will become unnecessary with v3 tables



907
908
909
910
911
912
913
914
915
916
917
918
919
920
921
922
# File 'lib/kramdown-rfc2629.rb', line 907

def clean_pcdatav3(parts) # hack, will become unnecessary with v3 tables
  clean = ''
  parts.each do |p|
    next if p.empty?
    if p == "<br />"
      p = "\u2028"        # XXX
    end
    d = REXML::Document.new("<foo>#{p}</foo>")
    t = REXML::XPath.each(d.root, "//text()").to_a.join
    if t != p
      warn "** simplified markup #{p.inspect} into #{t.inspect} in table heading"
    end
    clean << t
  end
  clean
end

#convert(el) ⇒ Object



398
399
400
401
402
403
404
405
406
407
408
409
410
# File 'lib/kramdown-rfc2629.rb', line 398

def convert(el)
  opts = el.options[:options]
  # warn "** tree opts #{opts.inspect}"
  if nested_ol_types = @options[:nested_ol_types]
    opts[:nested_ol_types] ||= nested_ol_types
    # warn "** tree opts out #{opts.inspect}"
  end
  indent = -INDENTATION
  if el.children[-1].type == :raw
    raw = convert1(el.children.pop, indent, opts)
  end
  "#{convert1(el, indent, opts)}#{end_sections(1, indent, el.options[:location])}#{raw}"
end

#convert1(el, indent, opts = {}) ⇒ Object



412
413
414
415
# File 'lib/kramdown-rfc2629.rb', line 412

def convert1(el, indent, opts = {})
  nopts = el.rfc2629_fix(opts)
  send("convert_#{el.type}", el, indent, nopts)
end

#convert_a(el, indent, opts) ⇒ Object



1165
1166
1167
1168
1169
1170
1171
1172
1173
1174
1175
1176
1177
1178
1179
1180
1181
1182
1183
1184
1185
1186
1187
1188
1189
1190
# File 'lib/kramdown-rfc2629.rb', line 1165

def convert_a(el, indent, opts)
  gi = el.attr.delete('gi')
  res = inner(el, indent, opts)
  target = el.attr['target']
  if target[0..1] == "{{"
    # XXX ignoring all attributes and content
    s = ::Kramdown::Converter::Rfc2629::process_markdown(target)
    # if res != '' && s[-2..-1] == '/>'
    #   if s =~ /\A<([-A-Za-z0-9_.]+) /
    #     gi ||= $1
    #   end
    #   s[-2..-1] = ">#{res}</#{gi}>"
    # end
    return s
  end
  if target[0] == "#"     # handle [](#foo) as xref as in RFC 7328
    el.attr['target'] = target = target[1..-1]
    if target.downcase == res.downcase
      res = ''            # get rid of raw anchors leaking through
    end
    gi ||= "xref"
  else
    gi ||= "eref"
  end
  "<#{gi}#{el_html_attributes(el)}>#{res}</#{gi}>"
end

#convert_abbreviation(el, indent, opts) ⇒ Object

XXX: This is wrong



1626
1627
1628
1629
1630
1631
1632
1633
1634
1635
1636
1637
1638
1639
1640
1641
1642
1643
1644
1645
1646
1647
1648
1649
1650
1651
1652
1653
1654
1655
1656
1657
1658
1659
1660
1661
1662
1663
1664
1665
1666
1667
1668
1669
1670
1671
1672
1673
1674
1675
1676
1677
1678
1679
1680
1681
1682
1683
1684
1685
1686
1687
1688
1689
1690
1691
# File 'lib/kramdown-rfc2629.rb', line 1626

def convert_abbreviation(el, indent, opts) # XXX: This is wrong
  if opts[:noabbrev]
    return el.value
  end

  value = el.value
  ix = value.gsub(/[\s\p{Z}]+/, " ")
  title = @root.options[:abbrev_defs][ix]
  if title.nil?
    warn "*** abbrev mismatch: value = #{value.inspect} ix = #{ix.inspect}"
  else
    title = nil if title.empty?
  end

  if title == "<bcp14>" && $options.v3
    return "<bcp14>#{value}</bcp14>"
  end

  hacked_value = value

  nobr = false
  if title && title =~ /\A<nobr>(\z|\s)/
    nobr = true
    _nobr, title = title.split(' ', 2)
    hacked_value = nobr_hack(value)
    if title.nil? || title.empty?
      return hacked_value # we have "exhausted" this abbrev -- suppress normal meaning
    end
  end

  if title && title[0] == "#"
    target, title = title.split(' ', 2)
    if target == "#"
      target = value
    else
      target = target[1..-1]
    end
  else
    target = nil
  end

  if item = title
    pairs = title.split(Parser::RFC2629Kramdown::IREF_START).each_slice(2).to_a
    replacement = pairs.map {|x,| s = x.strip; s unless s.empty?}.compact.join(" ")
    irefs = pairs.map {|_,x| x && [x]}.compact
    warn "@@@ ABBREV MISMATCH #{irefs}" if title.scan(Parser::RFC2629Kramdown::IREF_START) != irefs
    if irefs.empty?
      subitem = value
    else
      iref = irefs.map{|a,| iref_attr(a)}.join('')
    end
    unless replacement.empty?
      replacement = nobr_hack(replacement) if nobr # XXX this can break XML
      replacement = ::Kramdown::Converter::Rfc2629::process_markdown(replacement)
      hacked_value = replacement
    end
  else
    item = value
  end
  iref ||= "<iref#{html_attributes(item: item, subitem: subitem)}/>"
  if target
    "#{iref}<xref#{html_attributes(target: target, format: "none")}>#{hacked_value}</xref>"
  else
    "#{iref}#{hacked_value}"
  end
end

#convert_blank(el, indent, opts) ⇒ Object



429
430
431
# File 'lib/kramdown-rfc2629.rb', line 429

def convert_blank(el, indent, opts)
  "\n"
end

#convert_blockquote(el, indent, opts) ⇒ Object



862
863
864
865
866
867
868
869
870
871
872
873
874
875
876
877
878
879
880
# File 'lib/kramdown-rfc2629.rb', line 862

def convert_blockquote(el, indent, opts)
  text = inner(el, indent, opts)
  if $options.v3
    gi = el.attr.delete('gi')
    if gi && gi != 'ul'
      if RFCXML_SPAN_ELEMENTS === gi
        text.sub!(/\A\s*<t>(.*)<\/t>\s*\z/) {$1} # XXX unwrap inner text from block
        "#{' '*indent}<t><#{gi}#{el_html_attributes(el)}>#{text}</#{gi}></t>\n"
      else
        "#{' '*indent}<#{gi}#{el_html_attributes(el)}>\n#{text}#{' '*indent}</#{gi}>\n"
      end
    else
      "#{' '*indent}<ul#{el_html_attributes_with(el, {"empty" => 'true'})}><li>\n#{text}#{' '*indent}</li></ul>\n"
    end
  else
  text = "<t></t>" unless text =~ /</ # empty block quote
  "#{' '*indent}<t><list style='empty'#{el_html_attributes(el)}>\n#{text}#{' '*indent}</list></t>\n"
  end
end

#convert_br(el, indent, opts) ⇒ Object



1157
1158
1159
1160
1161
1162
1163
# File 'lib/kramdown-rfc2629.rb', line 1157

def convert_br(el, indent, opts)
  if $options.v3
    "<br />"
  else
    "<vspace />"
  end
end

#convert_codeblock(el, indent, opts) ⇒ Object



687
688
689
690
691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
750
751
752
753
754
755
756
757
758
759
760
761
762
763
764
765
766
767
768
769
770
771
772
773
774
775
776
777
778
779
780
781
782
783
784
785
786
787
788
789
790
791
792
# File 'lib/kramdown-rfc2629.rb', line 687

def convert_codeblock(el, indent, opts)
  # el.attr['anchor'] ||= saner_generate_id(el.value) -- no longer in 1.0.6
  result = el.value
  gi = el.attr.delete('gi')
  blockclass = el.attr.delete('class')
  if blockclass == 'language-tbreak'
    result = result.lines.map {|line| [line.chomp, 0]}
    spaceind = 0
    result.each_with_index {|pair, index|
      if pair[0] == ''
        result[spaceind][1] += 1
        pair[0] = nil unless index == spaceind
      else
        spaceind = index
      end
    }
    # $stderr.puts(result.inspect)
    result = result.map {|line, space|
      "<![CDATA[#{line.gsub(/^\s+/) {|s| "\u00A0" * s.size}}]]><vspace blankLines=\"#{space}\"/>" if line
    }.compact.join("\n")
    "#{' '*indent}<t>#{result}</t>\n"
  else
    artwork_attr = {}
    t = nil
    if blockclass
      classes = blockclass.split(' ')
      classes.each do |cl|
        if md = cl.match(/\Alanguage-(.*)/)
          t = artwork_attr["type"] = md[1] # XXX overwrite
        else
          $stderr.puts "*** Unimplemented codeblock class: #{cl}"
        end
      end
    end
    # compensate for XML2RFC idiosyncrasy by insisting on a blank line
    unless el.attr.delete('tight')
      result[0,0] = "\n" unless result[0,1] == "\n"
    end
    el.attr.each do |k, v|
      if md = k.match(/\A(?:artwork|sourcecode)-(.*)/)
        el.attr.delete(k)
        artwork_attr[md[1]] = v
      end
    end
    case t
    when "aasvg", "ditaa", "goat",
         "math", "math-asciitex", "mermaid",  "mscgen",
         "plantuml", "plantuml-utxt", "plantuml-ascii-art",
         "protocol", "protocol-aasvg", "protocol-goat",
         "railroad", "railroad-utf8"
      if gi
        warn "*** Can't set GI #{gi} for composite SVG artset"
      end
      result, result1 = memoize(:svg_tool_process, t,
                                artwork_attr.delete("svg-options"),
                                artwork_attr.delete("txt-options"),
                                result)
      retart = mk_artwork(artwork_attr, "ascii-art",
                          "<![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]>")
      if result1          # nest TXT in artset with SVG
        retsvg = mk_artwork(artwork_attr, "svg",
                            result1.sub(/.*?<svg/m, "<svg")) # , t == "mermaid")
        retart = "<artset>#{retsvg}#{retart}</artset>"
      end
      "#{' '*indent}<figure#{el_html_attributes(el)}>#{retart}</figure>\n"
    else
      gi ||= (
        if !$options.v3 || !t || ARTWORK_TYPES.include?(t)
          "artwork"
        elsif artwork_attr["align"]
          warn "** RFXCMLv3 does not yet allow setting align= for <sourcecode, falling back to <artwork"
          "artwork"
        else
          "sourcecode"
        end
      )
      loc_str =
        if anchor = el.attr['anchor']
          "##{anchor}"
        elsif lineno = el.options[:location]
          "#{correct_location(lineno)}"
        else
          "UNKNOWN"
        end
      preprocs = el.attr.delete("pre")
      checks = el.attr.delete("check")
      postprocs = el.attr.delete("post")
      case t
      when "cbor"
        warn "** There is no sourcecode-type “cbor”."
        warn "**   Do you mean “cbor-diag” (diagnostic notation)"
        warn "**   or “cbor-pretty” (annotated hex-dump)?"
      when "json", /\+json\z/
        checks ||= "json"
      when /\A(.*)-from-yaml\z/
        t = $1
        preprocs ||= "yaml2json"
      end
      preprocs = (preprocs || '').split("-")
      checks = (checks || '').split("-")
      postprocs = (postprocs || '').split("-")
      result = sourcecode_checkproc(preprocs, checks, postprocs, loc_str, result)
      "#{' '*indent}<figure#{el_html_attributes(el)}><#{gi}#{html_attributes(artwork_attr)}><![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]></#{gi}></figure>\n"
    end
  end
end

#convert_codespan(el, indent, opts) ⇒ Object



1465
1466
1467
1468
# File 'lib/kramdown-rfc2629.rb', line 1465

def convert_codespan(el, indent, opts)
  attrstring = el_html_attributes_with(el, {"style" => 'verb'})
  "<spanx#{attrstring}>#{escape_html(el.value)}</spanx>"
end

#convert_comment(el, indent, opts) ⇒ Object



1148
1149
1150
1151
1152
1153
1154
1155
# File 'lib/kramdown-rfc2629.rb', line 1148

def convert_comment(el, indent, opts)
## Don't actually output all those comments into the XML:
#        if el.options[:category] == :block
#          "#{' '*indent}<!-- #{el.value} -->\n"
#        else
#          "<!-- #{el.value} -->"
#        end
end

#convert_contact(el, indent, opts) ⇒ Object



1213
1214
1215
# File 'lib/kramdown-rfc2629.rb', line 1213

def convert_contact(el, indent, opts)
  "<contact#{el_html_attributes(el)}/>"
end

#convert_dd(el, indent, opts) ⇒ Object



1014
1015
1016
1017
1018
1019
1020
1021
1022
1023
1024
1025
1026
1027
1028
1029
1030
1031
1032
1033
1034
1035
1036
1037
# File 'lib/kramdown-rfc2629.rb', line 1014

def convert_dd(el, indent, opts)
  if $options.v3
    out = ''
    if !opts[:haddt]
      out ="#{' '*indent}<dt/>\n" # you can't make this one up
    end
    opts[:haddt] = false
    out << "#{' '*indent}<dd#{el_html_attributes(el)}>\n#{inner(el, indent, opts)}#{' '*indent}</dd>\n"
  else
  output = ' '*indent
  if @in_dt == 1
    @in_dt = 0
  else
    output << "<t#{el_html_attributes(el)}>"
  end
  res = inner(el, indent+INDENTATION, opts.merge(unpacked: true))
#        if el.children.empty? || el.children.first.options[:category] != :block
    output << res << (res =~ /\n\Z/ ? ' '*indent : '')
#        else                    FIXME: The latter case is needed for more complex cases
#          output << "\n" << res << ' '*indent
#        end
  output << "</t>\n"
  end
end

#convert_dl(el, indent, opts) ⇒ Object



985
986
987
988
989
990
991
992
993
994
995
996
997
998
# File 'lib/kramdown-rfc2629.rb', line 985

def convert_dl(el, indent, opts)
  if $options.v3
    if hangindent = el.attr.delete('hangIndent')
      el.attr['indent'] ||= hangindent # new attribute name wins
    end
    vspace = el.attr.delete('vspace')
    if vspace && !el.attr['newline']
      el.attr['newline'] = 'true'
    end
    "#{' '*indent}<dl#{el_html_attributes(el)}>\n#{inner(el, indent, opts.dup)}#{' '*indent}</dl>\n"
  else
    convert_ul(el, indent, opts)
  end
end

#convert_dt(el, indent, opts) ⇒ Object

SERIOUSLY BAD HACK:



1039
1040
1041
1042
1043
1044
1045
1046
1047
1048
1049
1050
1051
1052
1053
1054
1055
# File 'lib/kramdown-rfc2629.rb', line 1039

def convert_dt(el, indent, opts) # SERIOUSLY BAD HACK:
  if $options.v3
    out = ''
    if opts[:haddt]
      out ="#{' '*indent}<dd><t/></dd>\n" # you can't make this one up
    end
    opts[:haddt] = true
    out << "#{' '*indent}<dt#{el_html_attributes(el)}>#{inner(el, indent, opts)}</dt>\n"
  else
  close = "#{' '*indent}</t>\n" * @in_dt
  @in_dt = 1
  vspace = opts[:vspace]
  vspaceel = "<vspace blankLines='#{vspace}'/>" if vspace
  ht = escape_html(inner(el, indent, opts), :attribute) # XXX this may leave gunk
  "#{close}#{' '*indent}<t#{el_html_attributes(el)} hangText=\"#{ht}\">#{vspaceel}\n"
  end
end

#convert_em(el, indent, opts) ⇒ Object Also known as: convert_strong



1507
1508
1509
1510
1511
1512
1513
1514
1515
1516
# File 'lib/kramdown-rfc2629.rb', line 1507

def convert_em(el, indent, opts)
  if $options.v3
    gi = el.type
    "<#{gi}#{el_html_attributes(el)}>#{inner(el, indent, opts)}</#{gi}>"
  else
  attrstring = el_html_attributes_with(el, {"style" => EMPH[el.type]})
  span, irefs = clean_pcdata(inner_a(el, indent, opts))
  "<spanx#{attrstring}>#{span}</spanx>#{irefs}"
  end
end

#convert_entity(el, indent, opts) ⇒ Object



1519
1520
1521
# File 'lib/kramdown-rfc2629.rb', line 1519

def convert_entity(el, indent, opts)
  entity_to_str(el.value)
end

#convert_footnote(el, indent, opts) ⇒ Object

XXX: footnotes into crefs???



1470
1471
1472
1473
1474
1475
1476
1477
1478
1479
1480
1481
1482
1483
1484
1485
1486
1487
1488
1489
1490
1491
1492
1493
1494
1495
1496
1497
1498
# File 'lib/kramdown-rfc2629.rb', line 1470

def convert_footnote(el, indent, opts) # XXX: footnotes into crefs???
  # this would be more like xml2rfc v3:
  # "\n#{' '*indent}<cref>\n#{inner(el.value, indent, opts).rstrip}\n#{' '*indent}</cref>"
  content = inner(el.value, indent, opts).strip
  content = content.sub(/\A<t>(.*)<\/t>\z/m) {$1}
  name = ::Kramdown::Parser::RFC2629Kramdown.idref_cleanup(el.options[:name])
  o_name = name.dup
  while @footnote_names_in_use[name] do
    if name =~ /_\d+\z/
      name.succ!
    else
      name << "_1"
    end
  end
  @footnote_names_in_use[name] = true
  attrstring = el_html_attributes_with(el, {"anchor" => name})
  if $options.v3
    if o_name[-1] == "-"
      # Ignore HTML attributes.  Hmm.
      content
    else
      # do not indent span-level so we can stick to previous word.  Good?
      "<cref#{attrstring}>#{content}</cref>"
    end
  else
    content = escape_html(content, :text) # text only...
    "\n#{' '*indent}<cref#{attrstring}>#{content}</cref>"
  end
end

#convert_header(el, indent, opts) ⇒ Object



924
925
926
927
928
929
930
931
932
933
934
935
936
937
938
939
940
941
942
943
944
945
946
947
# File 'lib/kramdown-rfc2629.rb', line 924

def convert_header(el, indent, opts)
  # todo: handle appendix tags
  el = el.deep_clone
  options = @doc ? @doc.options : @options # XXX: 0.11 vs. 0.12
  if options[:auto_ids] && !el.attr['anchor']
    el.attr['anchor'] = saner_generate_id(el.options[:raw_text])
  end
  if $options.v3
    if sl = el.attr.delete('slugifiedName') # could do general name- play
      attrstring = html_attributes({'slugifiedName' => sl})
    end
    # noabbrev: true -- Workaround for https://github.com/ietf-tools/xml2rfc/issues/683
    nm = inner(el, indent, opts.merge(noabbrev: true))
    if ttl = el.attr['title']
      warn "*** Section has two titles: >>#{ttl}<< and >>#{nm}<<"
      warn "*** Do you maybe have a loose IAL?"
    end
    irefs = "<name#{attrstring}>#{nm}</name>" #
  else
  clean, irefs = clean_pcdata(inner_a(el, indent, opts))
  el.attr['title'] = clean
  end
  "#{end_sections(el.options[:level], indent, el.options[:location])}#{' '*indent}<section#{@sec_level += 1; el_html_attributes(el)}>#{irefs}\n"
end

#convert_hr(el, indent, opts) ⇒ Object

misuse for page break



949
950
951
# File 'lib/kramdown-rfc2629.rb', line 949

def convert_hr(el, indent, opts) # misuse for page break
  "#{' '*indent}<t><vspace blankLines='999' /></t>\n"
end

#convert_html_element(el, indent, opts) ⇒ Object



1059
1060
1061
1062
1063
1064
1065
1066
1067
1068
1069
1070
1071
1072
1073
1074
1075
1076
1077
1078
1079
1080
1081
1082
# File 'lib/kramdown-rfc2629.rb', line 1059

def convert_html_element(el, indent, opts)
  if el.value == "svg"    # do not meddle with id= in SVG
    opts = opts.merge({noanchor: true})
  end
  res = inner(el, indent, opts)
  if el.options[:category] == :span
    "<#{el.value}#{el_html_attributes(el)}" << (!res.empty? ? ">#{res}</#{el.value}>" : " />")
  else
    output = ''
    output << ' '*indent if !el.options[:parent_is_raw]
    output << "<#{el.value}#{el_html_attributes(el)}"
    if !res.empty? && el.options[:parse_type] != :block
      output << ">#{res}</#{el.value}>"
    elsif !res.empty?
      output << ">\n#{res}"  << ' '*indent << "</#{el.value}>"
    elsif HTML_TAGS_WITH_BODY.include?(el.value)
      output << "></#{el.value}>"
    else
      output << " />"
    end
    output << "\n" if el.options[:outer_element] || !el.options[:parent_is_raw]
    output
  end
end

#convert_img(el, indent, opts) ⇒ Object

misuse the tag!



1380
1381
1382
1383
1384
1385
1386
1387
1388
1389
1390
1391
1392
1393
1394
1395
1396
1397
1398
1399
1400
1401
1402
1403
1404
1405
1406
1407
1408
1409
1410
1411
1412
1413
1414
1415
1416
1417
1418
1419
1420
1421
1422
1423
1424
1425
1426
1427
1428
1429
1430
1431
1432
1433
1434
1435
1436
1437
1438
1439
1440
1441
1442
1443
1444
1445
1446
1447
1448
1449
1450
1451
1452
1453
1454
1455
1456
1457
1458
1459
1460
1461
1462
1463
# File 'lib/kramdown-rfc2629.rb', line 1380

def convert_img(el, indent, opts) # misuse the tag!
  if a = el.attr
    alt = a.delete('alt').strip
    alt = '' if alt == '!' # work around re-wrap uglyness
    if src = a.delete('src')
      a['target'] = src
    end
  end
  if alt == ":include:"   # Really bad misuse of tag...
    ann = el.attr.delete('ann')
    anchor = el.attr.delete('anchor') || (
      # not yet
      warn "*** missing anchor for '#{src}'"
      src
    )
    anchor = ::Kramdown::Parser::RFC2629Kramdown.idref_cleanup(anchor)
    anchor.gsub!('/', '_')              # should take out all illegals
    to_insert = ""
    src.scan(/(W3C|3GPP|[A-Z-]+)[.]?([A-Za-z_0-9.\(\)\/\+-]+)/) do |t, n|
      never_altproc = n.sub!(/^[.]/, "")
      fn = "reference.#{t}.#{n}.xml"
      sub, ttl, _can_anchor, altproc, always_altproc = XML_RESOURCE_ORG_MAP[t]
      ttl ||= KRAMDOWN_REFCACHETTL  # everything but RFCs might change a lot
      puts "*** Huh: #{fn}" unless sub
      if altproc && !never_altproc && (!KRAMDOWN_USE_TOOLS_SERVER || always_altproc)
        fn, url = altproc.call(fn, n)
      else
        url = "#{XML_RESOURCE_ORG_PREFIX}/#{sub}/#{fn}"
        fn = "alt-#{fn}" if never_altproc || KRAMDOWN_USE_TOOLS_SERVER
      end
      # if can_anchor # create anchor server-side for stand_alone: false
      #   url << "?anchor=#{anchor}"
      #   fn[/.xml$/] = "--anchor=#{anchor}.xml"
      # end
      to_insert = get_and_cache_resource(url, fn.gsub('/', '_'), ttl)
      to_insert.scrub! rescue nil # only do this for Ruby >= 2.1

      begin
        d = REXML::Document.new(to_insert)
        d.xml_decl.nowrite
        d.delete d.doctype
        d.context[:attribute_quote] = :quote  # Set double-quote as the attribute value delimiter
        d.root.attributes["anchor"] = anchor
        if t == "RFC" or t == "I-D"
          if KRAMDOWN_NO_TARGETS || !KRAMDOWN_KEEP_TARGETS
            d.root.attributes["target"] = nil
            REXML::XPath.each(d.root, "/reference/format") { |x|
              d.root.delete_element(x)
            }
          else
            REXML::XPath.each(d.root, "/reference/format") { |x|
              x.attributes["target"].sub!(%r{https?://www.ietf.org/internet-drafts/},
                                          %{https://www.ietf.org/archive/id/}) if t == "I-D"
            }
          end
        elsif t == "IANA"
          d.root.attributes["target"].sub!(%r{\Ahttp://www.iana.org/assignments/}, 'https://www.iana.org/assignments/')
        end
        if ann
          el = ::Kramdown::Converter::Rfc2629::process_markdown_to_rexml(ann).root
          el.name = "annotation"
          d.root.add_element(el)
        end
        to_insert = d.to_s
      rescue Exception => e
        warn "** Can't manipulate reference XML: #{e}"
        broken = true
        to_insert = nil
      end
      # this may be a bit controversial: Don't break the build if reference is broken
      if KRAMDOWN_OFFLINE || broken
        unless to_insert
          to_insert = "<reference anchor='#{anchor}'> <front> <title>*** BROKEN REFERENCE ***</title> <author> <organization/> </author> <date/> </front> </reference>"
          warn "*** KRAMDOWN_OFFLINE: Inserting broken reference for #{fn}"
        end
      else
        exit 66 unless to_insert # EX_NOINPUT
      end
    end
    to_insert
  else
    "<xref#{el_html_attributes(el)}>#{alt}</xref>"
  end
end

#convert_iref(el, indent, opts) ⇒ Object



1622
1623
1624
# File 'lib/kramdown-rfc2629.rb', line 1622

def convert_iref(el, indent, opts)
  iref_attr(el.attr['target'])
end

#convert_li(el, indent, opts) ⇒ Object



1000
1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
# File 'lib/kramdown-rfc2629.rb', line 1000

def convert_li(el, indent, opts)
  res_a = inner_a(el, indent, opts)
  if el.children.empty? || el.children.first.options[:category] == :span
    res = res_a.join('')
  else                    # merge multiple <t> elements
    res = res_a.select { |x|
      x.strip != ''
    }.map { |x|
      x.sub(/\A\s*<t>(.*)<\/t>\s*\Z/m) { $1}
    }.join("#{' '*indent}<vspace blankLines='1'/>\n").gsub(%r{(</list>)\s*<vspace blankLines='1'/>}) { $1 }.gsub(%r{<vspace blankLines='1'/>\s*(<list)}) { $1 }
  end
  "#{' '*indent}<t#{el_html_attributes(el)}>#{res}#{(res =~ /\n\Z/ ? ' '*indent : '')}</t>\n"
end

#convert_math(el, indent, opts) ⇒ Object

XXX: This is missing sup/sub support, which needs to be added



1561
1562
1563
1564
1565
1566
1567
1568
1569
1570
1571
1572
1573
1574
1575
1576
1577
1578
1579
1580
1581
1582
1583
1584
1585
1586
1587
1588
1589
1590
1591
1592
1593
1594
# File 'lib/kramdown-rfc2629.rb', line 1561

def convert_math(el, indent, opts) # XXX: This is wrong
  el = el.deep_clone
  if el.options[:category] == :block
    el.attr['artwork-type'] ||= ''
    el.attr['artwork-type'] += (el.attr['artwork-type'].empty? ? '' : ' ') + 'math'
    artwork_attr = {}
    el.attr.each do |k, v|
      if md = k.match(/\Aartwork-(.*)/)
        el.attr.delete(k)
        artwork_attr[md[1]] = v
      end
    end
    result, err, _s = Open3.capture3("tex2mail -noindent -ragged -by_par -linelength=69", stdin_data: el.value);
    # warn "*** tex2mail not in path?" unless s.success? -- doesn't have useful status
    capture_croak("tex2mail", err)
    "#{' '*indent}<figure#{el_html_attributes(el)}><artwork#{html_attributes(artwork_attr)}><![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]></artwork></figure>\n"

  else
    type = 'spanx'
    if $options.v3        # could use utftex if present
      content = escape_html(munge_latex(el.value).chomp, :text)
      attrstring = el_html_attributes_with(el, {"style" => 'none'})
    else
      warn "*** no support for inline math in XML2RFCv2"
      content = escape_html(el.value, :text)
      attrstring = el_html_attributes_with(el, {"style" => 'verb'})
    end
    if attrstring == ' style="none"'
      content
    else
      "<#{type}#{attrstring}>#{content}</#{type}>" # this might get a style=none...
    end
  end
end

#convert_ol(el, indent, opts) ⇒ Object



965
966
967
968
969
970
971
972
973
974
975
976
977
978
979
980
981
982
983
# File 'lib/kramdown-rfc2629.rb', line 965

def convert_ol(el, indent, opts)
  if @options[:ol_start_at_first_marker] and (first_list_marker =
                                              el.options[:first_list_marker])
    el.attr['start'] ||= first_list_marker[/\d+/]
  end
  nested_types = opts[:nested_ol_types] || ["1"]
  # warn "** ol opts #{opts.inspect} types #{nested_types.inspect}"
  if nested_attr = el.attr.delete('nestedOlTypes')
    nested_types = ::Kramdown::Options.parse(:nested_ol_types, nested_attr)
  end
  if nested_types = nested_types.dup
    # warn "** nested_types #{nested_types.inspect}"
    nested_here = nested_types.shift
    opts = opts.merge(nested_ol_types: nested_types << nested_here)
    el.attr['type'] ||= nested_here
    # warn "** actual ol type #{el.attr['type'].inspect}"
  end
  convert_ul(el, indent, opts)
end

#convert_p(el, indent, opts) ⇒ Object



437
438
439
440
441
442
443
# File 'lib/kramdown-rfc2629.rb', line 437

def convert_p(el, indent, opts)
  if (el.children.size == 1 && el.children[0].type == :img) || opts[:unpacked]
    inner(el, indent, opts) # Part of the bad reference hack
  else
    "#{' '*indent}<t#{el_html_attributes(el)}>#{inner(el, indent, opts)}</t>\n"
  end
end

#convert_raw(el, indent, opts) ⇒ Object



1500
1501
1502
1503
# File 'lib/kramdown-rfc2629.rb', line 1500

def convert_raw(el, indent, opts)
  end_sections(1, indent, el.options[:location]) +
  el.value + (el.options[:category] == :block ? "\n" : '')
end

#convert_root(el, indent, opts) ⇒ Object



1693
1694
1695
# File 'lib/kramdown-rfc2629.rb', line 1693

def convert_root(el, indent, opts)
  result = inner(el, indent, opts)
end

#convert_smart_quote(el, indent, opts) ⇒ Object



1540
1541
1542
# File 'lib/kramdown-rfc2629.rb', line 1540

def convert_smart_quote(el, indent, opts)
  entity_to_str(smart_quote_entity(el))
end

#convert_table(el, indent, opts) ⇒ Object

This only works for tables with headers



1106
1107
1108
1109
1110
# File 'lib/kramdown-rfc2629.rb', line 1106

def convert_table(el, indent, opts) # This only works for tables with headers
  alignment = el.options[:alignment].map { |al| ALIGNMENTS[al]}
  cols = (el.attr.delete("cols") || "").split(' ')
  "#{' '*indent}<texttable#{el_html_attributes(el)}>\n#{inner(el, indent, opts.merge(table_alignment: alignment, table_cols: cols))}#{' '*indent}</texttable>\n"
end

#convert_td(el, indent, opts) ⇒ Object Also known as: convert_th



1119
1120
1121
1122
1123
1124
1125
1126
1127
1128
1129
1130
1131
1132
1133
1134
1135
1136
1137
1138
1139
1140
1141
1142
1143
1144
1145
# File 'lib/kramdown-rfc2629.rb', line 1119

def convert_td(el, indent, opts)
  if alignment = opts[:table_alignment]
    alignment = alignment.shift
    if cols = opts[:table_cols].shift
      md = cols.match(/(\d*(|em|[%*]))([lrc])/)
      if md[1].to_i != 0
        widthval = md[1]
        widthval << "em" if md[2].empty?
        widthopt = "width='#{widthval}' "
      end
      alignment = COLS_ALIGN[md[3]] || :left
    end
  end
  if alignment
    xmlres = inner_a(el, indent, opts)
    if $options.v3
      res = clean_pcdatav3(xmlres)
    else
      res, irefs = clean_pcdata(xmlres)
      warn "*** lost markup #{irefs} in table heading" unless irefs.empty?
    end
    "#{' '*indent}<ttcol #{widthopt}align='#{alignment}'#{el_html_attributes(el)}>#{res.empty? ? "&#160;" : res}</ttcol>\n" # XXX need clean_pcdata
  else
    res = inner(el, indent, opts)
    "#{' '*indent}<c#{el_html_attributes(el)}>#{res.empty? ? "&#160;" : res}</c>\n"
  end
end

#convert_text(el, indent, opts) ⇒ Object



433
434
435
# File 'lib/kramdown-rfc2629.rb', line 433

def convert_text(el, indent, opts)
  escape_html(el.value, :text)
end

#convert_thead(el, indent, opts) ⇒ Object Also known as: convert_tbody, convert_tfoot, convert_tr



1112
1113
1114
# File 'lib/kramdown-rfc2629.rb', line 1112

def convert_thead(el, indent, opts)
  inner(el, indent, opts)
end

#convert_typographic_sym(el, indent, opts) ⇒ Object



1532
1533
1534
1535
1536
1537
1538
# File 'lib/kramdown-rfc2629.rb', line 1532

def convert_typographic_sym(el, indent, opts)
  if (result = @options[:typographic_symbols][el.value])
    escape_html(result, :text)
  else
    TYPOGRAPHIC_SYMS[el.value].map {|e| entity_to_str(e) }.join('')
  end
end

#convert_ul(el, indent, opts) ⇒ Object



955
956
957
958
959
960
961
962
963
# File 'lib/kramdown-rfc2629.rb', line 955

def convert_ul(el, indent, opts)
  opts = opts.merge(vspace: el.attr.delete('vspace'))
  attrstring = el_html_attributes_with(el, {"style" => STYLES[el.type]})
  if opts[:unpacked]
    "#{' '*indent}<list#{attrstring}>\n#{inner(el, indent, opts)}#{' '*indent}</list>\n"
    else
    "#{' '*indent}<t><list#{attrstring}>\n#{inner(el, indent, opts)}#{' '*indent}</list></t>\n"
  end
end

#convert_xml_comment(el, indent, opts) ⇒ Object Also known as: convert_xml_pi, convert_html_doctype



1084
1085
1086
1087
1088
1089
1090
1091
1092
1093
1094
1095
1096
1097
1098
1099
# File 'lib/kramdown-rfc2629.rb', line 1084

def convert_xml_comment(el, indent, opts)
  if el.value =~ /\A<\?line (([-+]?)[0-9]+)\?>\z/
    lineno = $1.to_i
    case $2
    when ''               # absolute
      @location_delta = lineno - el.options[:location]
    when '+', '-'         # correction (pre-scanning!)
      @location_correction += lineno
    end
  end
  if el.options[:category] == :block && !el.options[:parent_is_raw]
    ' '*indent + el.value + "\n"
  else
    el.value
  end
end

#convert_xref(el, indent, opts) ⇒ Object



1192
1193
1194
1195
1196
1197
1198
1199
1200
1201
1202
1203
1204
1205
1206
1207
1208
1209
1210
1211
# File 'lib/kramdown-rfc2629.rb', line 1192

def convert_xref(el, indent, opts)
  gi = el.attr.delete('gi')
  text = el.attr.delete('text')
  target = el.attr['target']
  if target[0] == "&"
    "#{target};"
  else
    if target =~ %r{\A\w+:(?://|.*@)}
      gi ||= "eref"
    else
      gi ||= "xref"
    end
    if text
      tail = ">#{Rfc2629::process_markdown(text)}</#{gi}>"
    else
      tail = "/>"
    end
    "<#{gi}#{el_html_attributes(el)}#{tail}"
  end
end

#correct_location(location) ⇒ Object



394
395
396
# File 'lib/kramdown-rfc2629.rb', line 394

def correct_location(location)
  location + @location_delta + @location_correction
end

#el_html_attributes(el) ⇒ Object



357
358
359
# File 'lib/kramdown-rfc2629.rb', line 357

def el_html_attributes(el)
  html_attributes(el.attr)
end

#el_html_attributes_with(el, defattr) ⇒ Object



360
361
362
# File 'lib/kramdown-rfc2629.rb', line 360

def el_html_attributes_with(el, defattr)
  html_attributes(defattr.merge(el.attr))
end

#end_sections(to_level, indent, location) ⇒ Object



882
883
884
885
886
887
888
889
890
891
892
893
# File 'lib/kramdown-rfc2629.rb', line 882

def end_sections(to_level, indent, location)
  if indent < 0
    indent = 0
  end
  if @sec_level >= to_level
    delta = (@sec_level - to_level)
    @sec_level = to_level
    "#{' '*indent}</section>\n" * delta
  else
    $stderr.puts "** #{correct_location(location)}: Bad section nesting: start heading level at 1 and increment by 1"
  end
end

#get_and_cache_resource(url, cachefile, tvalid = 7200, tn = Time.now) ⇒ Object

this is now slightly dangerous as multiple urls could map to the same cachefile



1260
1261
1262
1263
1264
1265
1266
1267
1268
1269
1270
1271
1272
1273
1274
1275
1276
1277
1278
1279
1280
1281
1282
1283
1284
1285
1286
1287
1288
1289
1290
1291
1292
1293
1294
1295
1296
1297
1298
1299
1300
1301
1302
1303
1304
1305
1306
1307
1308
1309
1310
1311
1312
1313
1314
1315
1316
1317
1318
1319
# File 'lib/kramdown-rfc2629.rb', line 1260

def get_and_cache_resource(url, cachefile, tvalid = 7200, tn = Time.now)
  fn = "#{REFCACHEDIR}/#{cachefile}"
  Dir.mkdir(REFCACHEDIR) unless Dir.exist?(REFCACHEDIR)
  f = File.stat(fn) rescue nil unless KRAMDOWN_REFCACHE_REFETCH
  if !KRAMDOWN_OFFLINE && (!f || tn - f.mtime >= tvalid)
    if f
      message = "renewing (stale by #{"%.1f" % ((tn-f.mtime)/86400)} days)"
      fetch_timeout = 10 # seconds, give up quickly if just renewing
    else
      message = "fetching"
      fetch_timeout = 60 # seconds; long timeout needed for Travis
    end
    $stderr.puts "#{fn}: #{message} from #{url}" unless KRAMDOWN_REFCACHE_QUIET
    if Array === url
      begin
        case url[0]
        when :DOI
          ref = get_doi(url[1])
          File.write(fn, ref)
        end
      rescue Exception => e
        warn "*** Error fetching #{url[0]} #{url[1].inspect}: #{e}"
      end
    elsif ENV["HAVE_WGET"]
      `cd #{REFCACHEDIR}; wget -t 3 -T #{fetch_timeout} -Nnv "#{url}"` # ignore errors if offline (hack)
      begin
        File.utime nil, nil, fn
      rescue Errno::ENOENT
        warn "Can't fetch #{url} -- is wget in path?"
      end
    else
      require 'open-uri'
      require 'socket'
      require 'openssl'
      require 'timeout'
      begin
        Timeout::timeout(fetch_timeout) do
          if $http
            begin         # belt and suspenders
              get_and_write_resource_persistently(url, fn)
            rescue Exception => e
              warn "*** Can't get with persistent HTTP: #{e}"
              get_and_write_resource(url, fn)
            end
          else
            get_and_write_resource(url, fn)
          end
        end
      rescue OpenURI::HTTPError, Errno::EHOSTUNREACH, Errno::ECONNREFUSED,
             SocketError, Timeout::Error => e
        warn "*** #{e} while fetching #{url}"
      end
    end
  end
  begin
    File.read(fn) # this blows up if no cache available after fetch attempt
  rescue Errno::ENOENT => e
    warn "*** #{e} for #{fn}"
  end
end

#get_and_write_resource(url, fn) ⇒ Object



1225
1226
1227
1228
1229
1230
1231
1232
1233
1234
1235
1236
1237
1238
1239
# File 'lib/kramdown-rfc2629.rb', line 1225

def get_and_write_resource(url, fn)
  options = {}
  if ENV["KRAMDOWN_DONT_VERIFY_HTTPS"]
    options[:ssl_verify_mode] = OpenSSL::SSL::VERIFY_NONE
  end             # workaround for OpenSSL on Windows...
  # URI.open(url, **options) do |uf|          # not portable to older versions
  OpenURI.open_uri(url, **options) do |uf|
    s = uf.read
    if uf.status[0] != "200"
      warn "*** Status code #{status} while fetching #{url}"
    else
      File.write(fn, s)
    end
  end
end

#get_and_write_resource_persistently(url, fn) ⇒ Object



1241
1242
1243
1244
1245
1246
1247
1248
1249
1250
1251
# File 'lib/kramdown-rfc2629.rb', line 1241

def get_and_write_resource_persistently(url, fn)
  t1 = Time.now
  response = $http.request(URI(url))
  if response.code != "200"
    raise "Status code #{response.code} while fetching #{url}"
  else
    File.write(fn, response.body)
  end
  t2 = Time.now
  warn "(#{"%.3f" % (t2 - t1)} s)" if KRAMDOWN_PERSISTENT_VERBOSE
end

#get_doi(refname) ⇒ Object



1253
1254
1255
1256
1257
# File 'lib/kramdown-rfc2629.rb', line 1253

def get_doi(refname)
  lit = doi_fetch_and_convert(refname, fuzzy: true)
  anchor = "DOI_#{refname.gsub("/", "_")}"
  KramdownRFC::ref_to_xml(anchor, lit)
end

#inner(el, indent, opts) ⇒ Object



425
426
427
# File 'lib/kramdown-rfc2629.rb', line 425

def inner(el, indent, opts)
  inner_a(el, indent, opts).join('')
end

#inner_a(el, indent, opts) ⇒ Object



417
418
419
420
421
422
423
# File 'lib/kramdown-rfc2629.rb', line 417

def inner_a(el, indent, opts)
  indent += INDENTATION
  el.children.map do |inner_el|
    nopts = inner_el.rfc2629_fix(opts)
    send("convert_#{inner_el.type}", inner_el, indent, nopts)
  end
end

#iref_attr(s) ⇒ Object



1599
1600
1601
1602
1603
1604
1605
1606
1607
# File 'lib/kramdown-rfc2629.rb', line 1599

def iref_attr(s)
  md = s.match(IREF_RE)
  attr = {
    item: md[2] || md[3],
    subitem: md[4] || md[5],
    primary: md[1] && 'true',
  }
  "<iref#{html_attributes(attr)}/>"
end

#memoize(meth, *args) ⇒ Object



552
553
554
555
556
557
558
559
560
561
562
563
564
565
# File 'lib/kramdown-rfc2629.rb', line 552

def memoize(meth, *args)
  require 'digest'
  Dir.mkdir(REFCACHEDIR) unless Dir.exist?(REFCACHEDIR)
  kdrfc_version = Gem.loaded_specs["kramdown-rfc2629"].version.to_s.gsub('.', '_') rescue "UNKNOWN"
  fn = "#{REFCACHEDIR}/kdrfc-#{kdrfc_version}-#{meth}-#{Digest::SHA256.hexdigest(Marshal.dump(args))[0...40]}.cache"
  begin
    out = Marshal.load(File.binread(fn))
  rescue StandardError => e
    # warn e.inspect
    out = method(meth).call(*args)
    File.binwrite(fn, Marshal.dump(out))
  end
  out
end

#mk_artwork(artwork_attr, typ, content, use_data = false) ⇒ Object



852
853
854
855
856
857
858
859
860
# File 'lib/kramdown-rfc2629.rb', line 852

def mk_artwork(artwork_attr, typ, content, use_data = false)
  if use_data
    warn "** using data URI for artwork type #{typ}"
    data = %{data:application/octet-stream;base64,#{[content].pack('m0')}}
    "<artwork #{html_attributes(artwork_attr.merge("type" => typ, "src" => data))}></artwork>"
  else
    "<artwork #{html_attributes(artwork_attr.merge("type" => typ))}>#{content}</artwork>"
  end
end

#munge_latex(s) ⇒ Object



1549
1550
1551
1552
1553
1554
1555
1556
1557
1558
# File 'lib/kramdown-rfc2629.rb', line 1549

def munge_latex(s)
  MATH_REPLACEMENTS.each do |o, n|
    s.gsub!(o, n)
  end
  MATH_COMBININGMARKS.each do |m, n|
    re = /\\#{m[1..-1]}\{(\X)\}/
    s.gsub!(re) { "#$1#{n}" }
  end
  s
end

#nobr_hack(s) ⇒ Object

replace this by actual <nobr> once that exists



1609
1610
1611
1612
1613
1614
1615
1616
1617
1618
1619
1620
# File 'lib/kramdown-rfc2629.rb', line 1609

def nobr_hack(s)          # replace this by actual <nobr> once that exists
  # https://github.com/ietf-tools/xml2rfc/blob/main/xml2rfc/utils.py#L42
  s.gsub(/([-\s\/])(?!\s)/) { case rep = $1
                             when /\A\s\z/
                               "\u00A0" # nbsp
                             when "-"
                               "\u2011" # nbhy -- XXX this might mangle dashes
                             else
                               "#{rep}\u2060"
                             end
  }
end

#saner_generate_id(value) ⇒ Object



445
446
447
# File 'lib/kramdown-rfc2629.rb', line 445

def saner_generate_id(value)
  generate_id(value).gsub(/-+/, '-')
end

#shell_prepare(opt) ⇒ Object



575
576
577
# File 'lib/kramdown-rfc2629.rb', line 575

def shell_prepare(opt)
  " " << opt.shellsplit.shelljoin
end

#sourcecode_checkproc(preprocs, checks, postprocs, loc_str, result) ⇒ Object



821
822
823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
845
846
847
848
849
850
# File 'lib/kramdown-rfc2629.rb', line 821

def sourcecode_checkproc(preprocs, checks, postprocs, loc_str, result)
  preprocs.each do |proc|
    result = sourcecode_proc(proc, loc_str, result)
  end if preprocs
  check_input = result
  checks.each do |check|
    case check
    when "skipheader"
      check_input = handle_artwork_sourcecode(check_input).sub(/.*?\n\n/m, '')
    when "json"
      # check for 8792; undo if needed:
      begin
        JSON.parse(handle_artwork_sourcecode(check_input))
      rescue => e
        err1 = "*** #{loc_str}: JSON isn't: #{JSON.dump(e.message[0..40])}\n"
        begin
          JSON.parse("{" << handle_artwork_sourcecode(check_input) << "}")
        rescue => e
          warn err1 << "***  not even with braces added around: #{JSON.dump(e.message[0..40])}"
        end
      end
    else
      warn "*** #{loc_str}: unknown check '#{check}'"
    end
  end if checks
  postprocs.each do |proc|
    result = sourcecode_proc(proc, loc_str, result)
  end if postprocs
  result
end

#sourcecode_proc(proc, loc_str, result) ⇒ Object



794
795
796
797
798
799
800
801
802
803
804
805
806
807
808
809
810
811
812
813
814
815
816
817
818
819
# File 'lib/kramdown-rfc2629.rb', line 794

def sourcecode_proc(proc, loc_str, result)
  case proc
  when "dedent"
    result = remove_indentation(result)
  when /\Afold(\d*)(left(\d*))?(dry)?\z/
    fold = [$1.to_i,            # col 0 for ''
            ($3.to_i if $2),    # left 0 for '', nil if no "left"
            $4]                 # dry
    result = fix_unterminated_line(fold8792_1(trim_empty_lines_around(result), *fold)) # XXX
  when /\Alines(\d*)\.\.(\.)?(\d*)\z/
    range = Range.new($1.empty? ? nil : $1.to_i, # compensate for
                      $3.empty? ? nil : $3.to_i, #   counting from 1
                      $2)
    result = result.lines[range].join
  when "yaml2json"
    begin
      y = YAML.safe_load(result, aliases: true, filename: loc_str)
      result = JSON.pretty_generate(y)
    rescue => e
      warn "*** #{loc_str}: YAML isn't: #{e.message}\n"
    end
  else
    warn "*** #{loc_str}: unknown proc '#{proc}'"
  end
  result
end

#svg_clean(s) ⇒ Object

expensive, risky



536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
# File 'lib/kramdown-rfc2629.rb', line 536

def svg_clean(s)          # expensive, risky
  d = REXML::Document.new(s)
  REXML::XPath.each(d.root, "//*[@shape-rendering]") { |x| x.attributes["shape-rendering"] = nil }  #; warn x.inspect }
  REXML::XPath.each(d.root, "//*[@text-rendering]") { |x| x.attributes["text-rendering"] = nil }  #; warn x.inspect  }
  REXML::XPath.each(d.root, "//*[@stroke]") { |x| x.attributes["stroke"] = svg_munch_color(x.attributes["stroke"], false) }
  REXML::XPath.each(d.root, "//*[@fill]") { |x| x.attributes["fill"] = svg_munch_color(x.attributes["fill"], true) }
  REXML::XPath.each(d.root, "//*[@id]") { |x| x.attributes["id"] = svg_munch_id(x.attributes["id"]) }
##      REXML::XPath.each(d.root, "//rect") { |x| x.attributes["style"] = "fill:none;stroke:black;stroke-width:1" unless x.attributes["style"] }
  # Fix for mermaid:
  REXML::XPath.each(d.root, "//polygon") { |x| x.attributes["rx"] = nil; x.attributes["ry"] = nil }
  d.to_s
rescue => detail
  warn "*** Can't clean SVG: #{detail}"
  d
end

#svg_clean_kgt(s) ⇒ Object



514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
# File 'lib/kramdown-rfc2629.rb', line 514

def svg_clean_kgt(s)
  d = REXML::Document.new(s)
  REXML::XPath.each(d.root, "/xmlns:svg", SVG_NAMESPACES) do |x|
    if (w = x.attributes["width"]) && (h = x.attributes["height"])
      x.attributes["viewBox"] = "0 0 %d %d" % [w, h]
    end
    if x.attributes["viewBox"]
      x.attributes["width"] = nil
      x.attributes["height"] = nil
    end
  end
  REXML::XPath.each(d.root, "//rect|//line|//path") do |x|
    x.attributes["fill"] = "none"
    x.attributes["stroke"] = "black"
    x.attributes["stroke-width"] = "1.5"
  end
  d.to_s
rescue => detail
  warn "*** Can't clean SVG: #{detail}"
  d
end

#svg_munch_color(c, fill) ⇒ Object



495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
# File 'lib/kramdown-rfc2629.rb', line 495

def svg_munch_color(c, fill)
  c = SVG_COLORS[c]
  case c
  when /\A#(..)(..)(..)\z/
    if hex_to_lin($1)*0.2126 + hex_to_lin($2)*0.7152 + hex_to_lin($3)*0.0722 >= B_W_THRESHOLD
      'white'
    else
      'black'
    end
  when 'none'
    'none' if fill        # delete for stroke
  else
    c
  end
end

#svg_munch_id(id) ⇒ Object



484
485
486
# File 'lib/kramdown-rfc2629.rb', line 484

def svg_munch_id(id)
  id.gsub(/[^-._A-Za-z0-9]/) {|x| "_%02X" % x.ord}
end

#svg_tool_process(t, svg_opt, txt_opt, result) ⇒ Object



581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
679
680
681
682
683
# File 'lib/kramdown-rfc2629.rb', line 581

def svg_tool_process(t, svg_opt, txt_opt, result)
  require 'tempfile'
  file = Tempfile.new("kramdown-rfc")
  file.write(result)
  file.close
  dont_clean = false
  dont_check = false
  svg_opt = shell_prepare(svg_opt) if svg_opt
  txt_opt = shell_prepare(txt_opt) if txt_opt
  case t
  when "protocol", "protocol-goat", "protocol-aasvg"
    cmdparm = result.lines.map(&:strip).select {|x| x != ''}.join(',')
    result, err, _s = Open3.capture3("protocol #{Shellwords.escape(cmdparm)}#{txt_opt}", stdin_data: '')
    if t == "protocol-goat"
      file.unlink
      file = Tempfile.new("kramdown-rfc")
      file.write(result)
      file.close
      result1, err, _s = Open3.capture3("goat#{svg_opt} #{file.path}", stdin_data: result);
      dont_clean = true
    elsif t == "protocol-aasvg"
      result1, err, _s = Open3.capture3("#{DEFAULT_AASVG}#{svg_opt}", stdin_data: result);
      dont_clean = true
      dont_check = true
    else
      result1 = nil
    end
  when "goat"
    result1, err, _s = Open3.capture3("goat#{svg_opt} #{file.path}", stdin_data: result);
    dont_clean = true
  when "aasvg"
    result1, err, _s = Open3.capture3("#{DEFAULT_AASVG}#{svg_opt}", stdin_data: result);
    dont_clean = true
    dont_check = true
  when "ditaa"        # XXX: This needs some form of option-setting
    result1, err, _s = Open3.capture3("ditaa #{file.path} --svg -o -#{svg_opt}", stdin_data: result);
  when "mscgen"
    result1, err, _s = Open3.capture3("mscgen -T svg -i #{file.path} -o -#{svg_opt}", stdin_data: result);
  when "mermaid"
    result1, err, _s = Open3.capture3("mmdc -i #{file.path}#{svg_opt}", stdin_data: result); #  -b transparent
    outpath = file.path + ".svg"
    result1 = File.read(outpath) rescue '' # don't die before providing error message
    File.unlink(outpath) rescue nil        # ditto
  when "plantuml", "plantuml-utxt", "plantuml-ascii-art"
    if t == "plantuml-ascii-art"
      result, ascii_art = result.split(/^~{3,} ascii-art\n/, 2)
      unless ascii_art
        warn "*** Didn't find ascii-art in plantuml-ascii-art #{result.inspect}"
        ascii_art = result.to_s
      end
    end
    plantuml = "@startuml\n#{result}\n@enduml"
    result1, err, _s = Open3.capture3("plantuml -pipe -tsvg#{svg_opt}", stdin_data: plantuml);
    case t
    when "plantuml-utxt"
      result, err1, _s = Open3.capture3("plantuml -pipe -tutxt#{txt_opt}", stdin_data: plantuml)
      err << err1.to_s
    when "plantuml-ascii-art"
      result = ascii_art
    end
  when "railroad", "railroad-utf8"
    result1, err1, _s = Open3.capture3("kgt -l abnf -e svg#{svg_opt}", stdin_data: result);
    result1 = svg_clean_kgt(result1); dont_clean = true
    result, err, _s = Open3.capture3("kgt -l abnf -e rr#{t == "railroad" ? "text" : "utf8"}#{txt_opt}",
                                      stdin_data: result);
    err << err1.to_s
  when "math", "math-asciitex"
    result1, err, _s = Open3.capture3("tex2svg --font STIX --speech=false#{svg_opt} #{Shellwords.escape(' ' << result)}");
    begin
      raise Errno::ENOENT if t == "math-asciitex"
      result, err1, s = Open3.capture3("utftex -m #{txt_opt}", stdin_data: result)
      if s.exitstatus != 0
        warn "** utftex: #{err1.inspect}"
        raise Errno::ENOENT
      end
    rescue Errno::ENOENT
      warn "** utftex not working, falling back to asciitex" unless t == "math-asciitex"
      result, err1, _s = Open3.capture3("asciitex -f #{file.path}#{txt_opt}")
    end
    err << err1
  end
  capture_croak(t, err)
  # warn ["text:", result.inspect]
  # warn ["svg:", result1.inspect]
  file.unlink
  if result1
    result1 = svg_clean(result1) unless dont_clean
    unless dont_check
      file = Tempfile.new("kramdown-rfc")
      file.write(result1)
      file.close
      result1, err, _s = Open3.capture3("svgcheck -qa #{file.path}");
      file.unlink
      # warn ["svgcheck:", result1.inspect]
      capture_croak("svgcheck", err)
    end
    if result1 == ''
      warn "*** could not create svg for #{result.inspect[0...20]}..."
      exit 65 # EX_DATAERR
    end
  end
  [result, result1]       # text, svg
end